@@ -1,14 +1,12 @@
 import astropy.units as u
-from astropy.units import Quantity
-
-from zodipy._component_label import ComponentLabel
 
+from ._ipd_comps import ComponentLabel
 
 T_0_DIRBE = 286  # K
 DELTA_DIRBE = 0.46686259861486573
 
-SPECTRUM_PLANCK = Quantity([100.0, 143.0, 217.0, 353.0, 545.0, 857.0], u.GHz)
-SPECTRUM_DIRBE = Quantity([1.25, 2.2, 3.5, 4.9, 12, 25, 60, 100, 140, 240], u.micron)
+SPECTRUM_PLANCK = u.Quantity([100.0, 143.0, 217.0, 353.0, 545.0, 857.0], u.GHz)
+SPECTRUM_DIRBE = u.Quantity([1.25, 2.2, 3.5, 4.9, 12, 25, 60, 100, 140, 240], u.micron)
 
 EMISSIVITY_PLANCK_13 = {
     ComponentLabel.CLOUD: (0.003, -0.014, 0.031, 0.168, 0.223, 0.301),
@@ -41,7 +39,6 @@ EMISSIVITY_ODEGARD = {
     ComponentLabel.BAND3: (0.50, 0.89, 1.22, 1.96, 2.81, 3.60),
 }
 
-
 EMISSIVITY_DIRBE = {
     ComponentLabel.CLOUD: (
         1.0,
@@ -193,7 +190,20 @@ ALBEDO_DIRBE = {
 }
 
 PHASE_FUNCTION_DIRBE = [
-    (-0.942, -0.527, -0.431, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0),
-    (0.121, 0.187, 0.172, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0),
-    (-0.165, -0.598, -0.633, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0),
+    (-0.94209999, -0.52670002, -0.4312, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0),
+    (0.1214, 0.18719999, 0.1715, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0),
+    (-0.1648, -0.59829998, -0.63330001, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0),
 ]
+
+SOLAR_IRRADIANCE_DIRBE = [
+    2.3405606e8,
+    1.2309874e8,
+    64292872,
+    35733824,
+    5763843.0,
+    1327989.4,
+    230553.73,
+    82999.336,
+    42346.605,
+    14409.608,
+] * (u.MJy / u.sr)
